@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -60,3 +60,11 @@ class NuitkaAssumptionError(AssertionError):
 
 class NuitkaPluginError(NuitkaErrorBase):
     pass
+
+
+class NuitkaCodeDeficit(NuitkaErrorBase):
+    pass
+
+
+class NuitkaNodeDesignError(Exception):
+    pass
