@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,6 +26,8 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
+from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
+
 from . import Contexts
 from .AsyncgenCodes import (
     generateMakeAsyncgenObjectCode,
@@ -75,6 +77,7 @@ from .ClassCodes import generateBuiltinSuperCode, generateSelectMetaclassCode
 from .CodeHelpers import setExpressionDispatchDict, setStatementDispatchDict
 from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
+    generateBuiltinIssubclassCode,
     generateComparisonExpressionCode,
 )
 from .ConditionalCodes import (
@@ -133,6 +136,7 @@ from .FrameCodes import (
 from .FunctionCodes import (
     generateFunctionCallCode,
     generateFunctionCreationCode,
+    generateFunctionErrorStrCode,
     generateFunctionOutlineCode,
     getExportScopeCode,
     getFunctionCode,
@@ -219,7 +223,7 @@ from .ReturnCodes import (
     generateGeneratorReturnValueCode,
     generateReturnCode,
     generateReturnConstantCode,
-    generateReturnedValueRefCode,
+    generateReturnedValueCode,
 )
 from .SetCodes import (
     generateBuiltinFrozensetCode,
@@ -231,7 +235,9 @@ from .SetCodes import (
 )
 from .SliceCodes import (
     generateAssignmentSliceCode,
-    generateBuiltinSliceCode,
+    generateBuiltinSlice1Code,
+    generateBuiltinSlice2Code,
+    generateBuiltinSlice3Code,
     generateDelSliceCode,
     generateSliceLookupCode,
 )
@@ -414,7 +420,6 @@ def generateModuleCode(module, data_filename):
 
     context = Contexts.PythonModuleContext(
         module=module,
-        # TODO: Have output filename already before generating code.
         data_filename=data_filename,
     )
 
@@ -424,8 +429,12 @@ def generateModuleCode(module, data_filename):
     function_body_codes = []
 
     for function_body in module.getUsedFunctions():
-        # Empty functions get no code.
-        if function_body.getBody() is None:
+        # Constant function returners get no code.
+        (
+            is_constant_returning,
+            _constant_return_value,
+        ) = function_body.getConstantReturnValue()
+        if is_constant_returning:
             continue
 
         function_code, function_decl = generateFunctionBodyCode(
@@ -455,7 +464,13 @@ def generateModuleCode(module, data_filename):
 
         function_decl_codes.append(function_decl)
 
-    return getModuleCode(module, function_decl_codes, function_body_codes, context)
+    return getModuleCode(
+        module=module,
+        function_decl_codes=function_decl_codes,
+        function_body_codes=function_body_codes,
+        module_const_blob_name=deriveModuleConstantsBlobName(data_filename),
+        context=context,
+    )
 
 
 def generateHelpersCode():
@@ -478,7 +493,9 @@ setExpressionDispatchDict(
         "EXPRESSION_ATTRIBUTE_CHECK": generateAttributeCheckCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP": generateAttributeLookupCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP_SPECIAL": generateAttributeLookupSpecialCode,
-        "EXPRESSION_BUILTIN_SLICE": generateBuiltinSliceCode,
+        "EXPRESSION_BUILTIN_SLICE3": generateBuiltinSlice3Code,
+        "EXPRESSION_BUILTIN_SLICE2": generateBuiltinSlice2Code,
+        "EXPRESSION_BUILTIN_SLICE1": generateBuiltinSlice1Code,
         "EXPRESSION_BUILTIN_HASH": generateBuiltinHashCode,
         "EXPRESSION_BUILTIN_ID": generateBuiltinIdCode,
         "EXPRESSION_BUILTIN_COMPILE": generateBuiltinCompileCode,
@@ -529,6 +546,7 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_GLOBALS": generateBuiltinGlobalsCode,
         "EXPRESSION_BUILTIN_SUPER": generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
+        "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
         "EXPRESSION_BUILTIN_VARS": generateBuiltinVarsCode,
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
@@ -560,8 +578,11 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_TRUE_REF": generateConstantTrueReferenceCode,
         "EXPRESSION_CONSTANT_FALSE_REF": generateConstantFalseReferenceCode,
         "EXPRESSION_CONSTANT_STR_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_STR_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_UNICODE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_UNICODE_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_BYTES_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_BYTES_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_INT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_LONG_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_FLOAT_REF": generateConstantReferenceCode,
@@ -571,6 +592,7 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_DICT_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_EMPTY_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TUPLE_MUTABLE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_LIST_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_LIST_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_SET_REF": generateConstantReferenceCode,
@@ -602,10 +624,12 @@ setExpressionDispatchDict(
         "EXPRESSION_DICT_OPERATION_NOT_IN": generateDictOperationInCode,
         "EXPRESSION_FUNCTION_CREATION": generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL": generateFunctionCallCode,
+        "EXPRESSION_FUNCTION_ERROR_STR": generateFunctionErrorStrCode,
         "EXPRESSION_IMPORT_MODULE_HARD": generateImportModuleHardCode,
         "EXPRESSION_IMPORT_MODULE_NAME_HARD": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORT_NAME": generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND": generateListOperationExtendCode,
+        "EXPRESSION_LIST_OPERATION_EXTEND_FOR_UNPACK": generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_POP": generateListOperationPopCode,
         "EXPRESSION_MODULE_ATTRIBUTE_FILE_REF": generateModuleAttributeFileCode,
         "EXPRESSION_MODULE_ATTRIBUTE_NAME_REF": generateModuleAttributeCode,
@@ -656,7 +680,6 @@ setExpressionDispatchDict(
         "EXPRESSION_OUTLINE_FUNCTION": generateFunctionOutlineCode,
         # TODO: Rename to make more clear it is an outline
         "EXPRESSION_CLASS_BODY": generateFunctionOutlineCode,
-        "EXPRESSION_RETURNED_VALUE_REF": generateReturnedValueRefCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP": generateSubscriptLookupCode,
         "EXPRESSION_SLICE_LOOKUP": generateSliceLookupCode,
         "EXPRESSION_SET_OPERATION_UPDATE": generateSetOperationUpdateCode,
@@ -704,6 +727,7 @@ setStatementDispatchDict(
         "STATEMENT_RETURN_FALSE": generateReturnConstantCode,
         "STATEMENT_RETURN_NONE": generateReturnConstantCode,
         "STATEMENT_RETURN_CONSTANT": generateReturnConstantCode,
+        "STATEMENT_RETURN_RETURNED_VALUE": generateReturnedValueCode,
         "STATEMENT_GENERATOR_RETURN": generateGeneratorReturnValueCode,
         "STATEMENT_GENERATOR_RETURN_NONE": generateGeneratorReturnNoneCode,
         "STATEMENT_CONDITIONAL": generateBranchCode,
