@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -46,9 +46,12 @@ def getLineNumberUpdateCode(context):
 
 
 def getErrorLineNumberUpdateCode(context):
-    _exception_type, _exception_value, _exception_tb, exception_lineno = (
-        context.variable_storage.getExceptionVariableDescriptions()
-    )
+    (
+        _exception_type,
+        _exception_value,
+        _exception_tb,
+        exception_lineno,
+    ) = context.variable_storage.getExceptionVariableDescriptions()
 
     lineno_value = getCurrentLineNumberCode(context)
 
