@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -53,6 +53,8 @@ class StatementAssignmentAttribute(StatementChildrenHavingBase):
     and what value it will be.
     """
 
+    __slots__ = ("attribute_name",)
+
     kind = "STATEMENT_ASSIGNMENT_ATTRIBUTE"
 
     named_children = ("source", "expression")
@@ -72,9 +74,6 @@ class StatementAssignmentAttribute(StatementChildrenHavingBase):
     def getAttributeName(self):
         return self.attribute_name
 
-    def setAttributeName(self, attribute_name):
-        self.attribute_name = attribute_name
-
     def computeStatement(self, trace_collection):
         result, change_tags, change_desc = self.computeStatementSubExpressions(
             trace_collection=trace_collection
@@ -90,7 +89,8 @@ class StatementAssignmentAttribute(StatementChildrenHavingBase):
             trace_collection=trace_collection,
         )
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "attribute assignment statement"
 
 
@@ -121,9 +121,6 @@ class StatementDelAttribute(StatementChildHavingBase):
     def getAttributeName(self):
         return self.attribute_name
 
-    def setAttributeName(self, attribute_name):
-        self.attribute_name = attribute_name
-
     def computeStatement(self, trace_collection):
         result, change_tags, change_desc = self.computeStatementSubExpressions(
             trace_collection=trace_collection
@@ -138,7 +135,8 @@ class StatementDelAttribute(StatementChildHavingBase):
             trace_collection=trace_collection,
         )
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "attribute del statement"
 
 
@@ -163,25 +161,23 @@ class ExpressionAttributeLookup(ExpressionChildHavingBase):
     def getAttributeName(self):
         return self.attribute_name
 
-    def setAttributeName(self, attribute_name):
-        self.attribute_name = attribute_name
-
     def getDetails(self):
-        return {"attribute_name": self.getAttributeName()}
+        return {"attribute_name": self.attribute_name}
 
     def computeExpression(self, trace_collection):
         return self.subnode_expression.computeExpressionAttribute(
             lookup_node=self,
-            attribute_name=self.getAttributeName(),
+            attribute_name=self.attribute_name,
             trace_collection=trace_collection,
         )
 
     def mayRaiseException(self, exception_type):
         return self.subnode_expression.mayRaiseExceptionAttributeLookup(
-            exception_type=exception_type, attribute_name=self.getAttributeName()
+            exception_type=exception_type, attribute_name=self.attribute_name
         )
 
-    def isKnownToBeIterable(self, count):
+    @staticmethod
+    def isKnownToBeIterable(count):
         # TODO: Could be known. We would need for computeExpressionAttribute to
         # either return a new node, or a decision maker.
         return None
@@ -201,7 +197,7 @@ class ExpressionAttributeLookupSpecial(ExpressionAttributeLookup):
     def computeExpression(self, trace_collection):
         return self.subnode_expression.computeExpressionAttributeSpecial(
             lookup_node=self,
-            attribute_name=self.getAttributeName(),
+            attribute_name=self.attribute_name,
             trace_collection=trace_collection,
         )
 
@@ -217,8 +213,6 @@ class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_GETATTR"
 
     named_children = ("expression", "name", "default")
-    getAttribute = ExpressionChildrenHavingBase.childGetter("name")
-    getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
     def __init__(self, expression, name, default, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -230,10 +224,10 @@ class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
-        default = self.getDefault()
+        default = self.subnode_default
 
         if default is None or not default.mayHaveSideEffects():
-            attribute = self.getAttribute()
+            attribute = self.subnode_name
 
             attribute_name = attribute.getStringValue()
 
@@ -277,8 +271,6 @@ class ExpressionBuiltinSetattr(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SETATTR"
 
     named_children = ("expression", "attribute", "value")
-    getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
-    getValue = ExpressionChildrenHavingBase.childGetter("value")
 
     def __init__(self, expression, name, value, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -298,7 +290,6 @@ class ExpressionBuiltinHasattr(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_HASATTR"
 
     named_children = ("expression", "attribute")
-    getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
 
     def __init__(self, expression, name, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -313,7 +304,7 @@ class ExpressionBuiltinHasattr(ExpressionChildrenHavingBase):
         source = self.subnode_expression
 
         if source.isCompileTimeConstant():
-            attribute = self.getAttribute()
+            attribute = self.subnode_attribute
 
             attribute_name = attribute.getStringValue()
 
@@ -391,7 +382,8 @@ class ExpressionAttributeCheck(ExpressionChildHavingBase):
 
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
     def getAttributeName(self):
