@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,7 +20,6 @@
 """
 
 import marshal
-from logging import info
 
 from nuitka import Options
 from nuitka.importing.ImportCache import (
@@ -30,18 +29,19 @@ from nuitka.importing.ImportCache import (
 from nuitka.ModuleRegistry import replaceRootModule
 from nuitka.nodes.ModuleNodes import makeUncompiledPythonModule
 from nuitka.plugins.Plugins import Plugins
+from nuitka.Tracing import inclusion_logger
 
 
 def demoteCompiledModuleToBytecode(module):
-    """ Demote a compiled module to uncompiled (bytecode).
-
-    """
+    """Demote a compiled module to uncompiled (bytecode)."""
 
     full_name = module.getFullName()
     filename = module.getCompileTimeFilename()
 
     if Options.isShowProgress():
-        info("Demoting module '%s' to bytecode from '%s'." % (full_name, filename))
+        inclusion_logger.info(
+            "Demoting module '%s' to bytecode from '%s'." % (full_name, filename)
+        )
 
     source_code = module.getSourceCode()
 
