@@ -31,6 +31,7 @@ from nuitka.containers.oset import OrderedSet
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
 from nuitka.utils.FileOperations import getFileContentByLine
+from nuitka.utils.Importing import getSharedLibrarySuffix
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.SharedLibraries import getPyWin32Dir, locateDLL
 from nuitka.utils.Utils import getOS, isWin32Windows
@@ -111,7 +112,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 "_QOpenGLFunctions_2_1",
                 "_QOpenGLFunctions_4_1_Core",
             ):
-                yield getChildNamed(".QtCore")
+                yield getChildNamed("QtCore")
 
             if child in (
                 "QtDeclarative",
@@ -127,14 +128,14 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 "QtWebSockets",
                 "QtWebEngineWidgets",
             ):
-                yield getChildNamed(".QtNetwork")
+                yield getChildNamed("QtNetwork")
 
             if child == "QtWebEngineWidgets":
-                yield getChildNamed(".QtWebEngineCore")
-                yield getChildNamed(".QtWebChannel")
-                yield getChildNamed(".QtPrintSupport")
+                yield getChildNamed("QtWebEngineCore")
+                yield getChildNamed("QtWebChannel")
+                yield getChildNamed("QtPrintSupport")
             elif child == "QtScriptTools":
-                yield getChildNamed(".QtScript")
+                yield getChildNamed("QtScript")
             elif child in (
                 "QtWidgets",
                 "QtDeclarative",
@@ -156,7 +157,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 "_QOpenGLFunctions_2_1",
                 "_QOpenGLFunctions_4_1_Core",
             ):
-                yield getChildNamed(".QtGui")
+                yield getChildNamed("QtGui")
 
             if full_name in (
                 "PyQt5.QtDesigner",
@@ -1489,6 +1490,18 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
 
             return result
 
+        elif full_name == "zmq.libzmq" and isWin32Windows():
+            # TODO: Very strange thing for zmq on Windows, needs the .pyd file in wrong dir too. Have
+            # this done in a dedicated form somewhere.
+            shutil.copyfile(
+                os.path.join(dist_dir, "zmq\\libzmq.pyd"),
+                os.path.join(
+                    dist_dir, "libzmq" + getSharedLibrarySuffix(preferred=True)
+                ),
+            )
+
+            return ()
+
         return ()
 
     unworthy_namespaces = (
